class WindowNavigation:
    """
    The Window Navigation Manager.

    :param current_url: The url to initially display.
    """

    def __init__(self, current_url = "/"):
        self.__homepage = None
        self.__previous = None
        self.__location = current_url

    def setdefault(self, url):
        """Sets the homepage."""
        self.__homepage = url
    
    @property
    def homepage(self):
        return self.__homepage

    @property
    def previous(self):
        return self.__previous

    @property
    def location(self):
        return self.__location

    @location.setter
    def location(self, new_url):
        self.__previous = None
        self.__location = new_url


import warnings

global __RALIUM_WARNING_MESSAGES__
__RALIUM_WARNING_MESSAGES__ = True

class WindowLoadError(Exception): pass
class BridgeEventError(Exception): pass
class FilePathLimitError(Exception): pass
class RegistryNotFoundError(Exception): pass

class WebFunctionApiError(Exception): pass
class WebFolderNotFoundError(Exception): pass
class WebRoutesNotFoundError(Exception): pass
class WebFolderDirectoryError(Exception): pass
class WebRoutesDirectoryError(Exception): pass

class WebHookNotFoundError(Exception): pass
class WebHookHomepageError(Exception): pass

class FrozenMemberExistsError(Exception): pass

class FileNotFoundWarning(Warning): pass

def DisableWarnings():
    """
    Disables Ralium warnings from being displayed.
    """

    global __RALIUM_WARNING_MESSAGES__
    __RALIUM_WARNING_MESSAGES__ = False

def warn(message, warning, major = False):
    if major:
        return warnings.warn(f"[Ralium]: {message}", warning)
    print(f"[Ralium] {warning.__name__}: {message}")


from ralium.webpage import WebHook
from ralium.config import WindowConfig
from ralium.window import Window

from ralium._util import (
    FunctionType,
    FilePathStr, 
    DirPathStr, 
    Callable,
    _RT
)

def _check_web_folder(webfolder: FilePathStr) -> None: ...
def _check_web_routes(webroutes: DirPathStr) -> None: ...
def _collect_global_css_files(webfolder: DirPathStr) -> list[FilePathStr]: ...
def _get_registry_contents(pyfile: FilePathStr) -> list[FunctionType]: ...
def _collect_webhooks(webfolder: DirPathStr, webroutes: DirPathStr) -> list[WebHook]: ...

class Module:
    _module_api_class: bool

    def __init__(self, *functions: tuple[FunctionType], **named_functions: dict[str, FunctionType]) -> None: ...

def create_window(webfolder: DirPathStr, config: WindowConfig | None = None) -> Window: ...
def register() -> Callable[..., _RT]: ...
def namespace(name: str, *functions: tuple[FunctionType], **named_functions: dict[str, FunctionType]) -> None: ...
def wrap(function: FunctionType) -> Callable[..., _RT]: ...


from typing import TypeAlias, TypeVar, Callable, Any, Self, Literal
from types import FunctionType, ModuleType
import http.server

__all__ = ["BasicHTTPServer", "_get_path_limit", "_get_path", "_is_markup_filelike"]

_RT = TypeVar("_RT") # Return Type
ClassType: TypeAlias = object
DirPathStr: TypeAlias = str
FilePathStr: TypeAlias = str

class BasicHTTPServer(http.server.SimpleHTTPRequestHandler): ...

def _get_path_limit_winreg() -> int: ...
def _get_path_limit() -> int: ...
def _get_path(path: str) -> str: ...
def _is_markup_filelike(markup: str) -> bool: ...


class WindowNavigation:
    def __init__(self, current_url = "/") -> None: ...
    def setdefault(self, url: str) -> None: ... 
    @property
    def homepage(self) -> str | None: ...
    @property
    def previous(self) -> str | None: ...
    @property
    def location(self) -> str | None: ...
    @location.setter
    def location(self, new_url: str) -> None: ...


from ralium.window import Window
from typing import Iterator
import os

HTML_TEMPLATE: str

class FileReader:
    def __init__(self, file: str | os.PathLike, encoding: str) -> None: ...
    @property
    def content(self) -> str: ...
    def _read(self) -> None: ...

class CSSReader:
    def __init__(self, *files: list[str | os.PathLike], encoding: str) -> None: ...
    @property
    def content(self) -> str: None

class WebHookFunction:
    def __new__(cls, function: function, window: Window) -> function: ...

class WebHook:
    def __init__(self, 
        url: str, 
        html: str | os.PathLike, 
        css: str | list[str] | None = None, 
        functions: list[function] | None = None, 
        homepage: bool = False, 
        encoding: str = "UTF-8"
    ) -> None: ...
    def __repr__(self) -> str: ...
    
    def _get_html(self) -> None: ...
    def _get_css(self) -> None: ...
    def _wrap_functions(self) -> None: ...
    def set_window(self, window: Window) -> None: ...

class WebHookDict(dict):
    def __init__(self, webhooks: list[WebHook]) -> None: ...
    def __repr__(self) -> str: ...
    def __iter__(self) -> Iterator[tuple]: ...
    def get(self, url: str) -> WebHook: ...